@@ -11,11 +11,12 @@ use Carp qw(confess);
 use Any::Moose;
 
 use AnyEvent::Net::Curl::Const;
+use Time::HiRes qw(time);
 
-our $VERSION = '0.029'; # VERSION
+our $VERSION = '0.030'; # VERSION
 
 
-has stamp       => (is => 'rw', isa => 'Int', default => time);
+has stamp       => (is => 'ro', isa => 'Num', default => time, writer => 'set_stamp');
 
 
 has stats       => (
@@ -58,7 +59,7 @@ sub sum {
                 : $from->getinfo(AnyEvent::Net::Curl::Const::info($type));
     }
 
-    $self->stamp(time);
+    $self->set_stamp(time);
 
     return 1;
 }
@@ -81,7 +82,7 @@ AnyEvent::Net::Curl::Queued::Stats - Connection statistics for AnyEvent::Net::Cu
 
 =head1 VERSION
 
-version 0.029
+version 0.030
 
 =head1 SYNOPSIS
 
